CREATE OR REPLACE FUNCTION partition_maintenance_function()
RETURNS INTEGER AS $$
DECLARE
_num_tables INTEGER;
_keep_tables INTEGER;
_max_locks INTEGER;
_result RECORD;
_deleted INTEGER;
_to_delete INTEGER;
BEGIN
	_max_locks := (SELECT setting FROM pg_settings WHERE name = 'max_locks_per_transaction');
	/*
	 * Make sure we have room for for pgsql statistics
	 * updates as well as the select query below
	 */
	_max_locks = _max_locks - 2;
	_keep_tables := (SELECT keep_in_db+1 FROM settings);
	_num_tables := (SELECT COUNT(1) FROM information_schema.tables
		WHERE table_catalog = 'op5lsu' AND table_name LIKE E'messages\_%');
	IF _num_tables <= _keep_tables THEN
		RETURN 0;
	END IF;
	_to_delete = _num_tables - _keep_tables;
	IF _max_locks < _to_delete THEN
		_to_delete = _max_locks;
	END IF;

	_deleted = 0;
	/*
	 * The ORDER BY and LIMIT here are crucial, as we'll simply
	 * delete all returned tables and we rely on postgresql to
	 * sort them for us so we delete the oldest ones first.
	 * Fortunately, the format lends itself nicely to that.
	 */
	FOR _result IN (SELECT table_name from information_schema.tables
		WHERE table_catalog = 'op5lsu' AND table_name LIKE E'messages\_%'
		ORDER BY table_name ASC LIMIT _to_delete) LOOP
		EXECUTE 'DROP TABLE ' || quote_ident(_result.table_name);
		_deleted = _deleted + 1;
	END LOOP;
	RETURN _deleted;
END
$$
LANGUAGE plpgsql;

UPDATE service SET version = 34;
